# Aurora

[![Kotlin Version](https://img.shields.io/badge/kotlin-1.5.21-blue.svg)](https://kotlinlang.org)
[![API](https://img.shields.io/badge/API-21%2B-brightgreen.svg?style=flat)](https://android-arsenal.com/api?level=24)
[![GitHub release (including pre-releases)](https://img.shields.io/github/v/release/FunkyMuse/Aurora?include_prereleases)](https://github.com/FunkyMuse/Aurora/releases/latest)
![GitHub Repo Size](https://img.shields.io/github/repo-size/FunkyMuse/Aurora)

### If my noble work has helped you, consider becoming a [![Sponsor me](https://img.shields.io/badge/sponsor-30363D?style=for-the-badge&logo=GitHub-Sponsors&logoColor=#EA4AAA)](https://github.com/sponsors/FunkyMuse/).

This is a non-official [Library Genesis](https://libgen.rs/) mobile client.

The project is completely independent and Open Source. 
The developer has no relation with [Library Genesis](https://libgen.rs/).

Publishing a copy of this application on the Play Store will only get your account banned.

## Features
- Search books by title, author, and other parameters
- See detailed info about the book (pages, size, etc)
- Switch between light and dark theme
- Dark mode/light mode
- Built with many amazing open source libraries such as ([KAHelpers](https://github.com/FunkyMuse/KAHelpers), [Crashy](https://github.com/FunkyMuse/Crashy), [Accompanist](https://github.com/google/accompanist), [Composed](https://github.com/FunkyMuse/Composed))
- and many more...

## Download
[![Get it on GitHub releases](https://i.ibb.co/q0mdc4Z/get-it-on-github.png)](https://github.com/FunkyMuse/Aurora/releases/latest)
[![Get it on F-Droid](https://fdroid.gitlab.io/artwork/badge/get-it-on.png)](https://f-droid.org/en/packages/com.funkymuse.aurora/)

## Screenshots

#### Light
<img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_5.png" width="33%"> </img><img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_6.png" width="33%"> </img><img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_7.png" width="33%"></img> <img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_8.png" width="33%"></img> 

#### Dark
<img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_1.png" width="33%"></img> <img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_2.png" width="33%"></img> <img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_3.png" width="33%"></img> <img src="https://raw.githubusercontent.com/FunkyMuse/Aurora/master/fastlane/metadata/android/en-US/images/phoneScreenshots/screen_4.png" width="33%"></img>


## Built With 🛠

Some of the popular libraries and MVVM clean architecture used with Room database as a source

* [Kotlin](https://kotlinlang.org/) - First class and official programming language for Android development.

* [Coroutines](https://kotlinlang.org/docs/reference/coroutines-overview.html) - Threads on steroids for Kotlin
* [Flow](https://kotlin.github.io/kotlinx.coroutines/kotlinx-coroutines-core/kotlinx.coroutines.flow/-flow/) - A cold asynchronous data stream that sequentially emits values and completes normally or with an exception.
* [StateFlow](https://kotlin.github.io/kotlinx.coroutines/kotlinx-coroutines-core/kotlinx.coroutines.flow/-state-flow/) - A live data replacement

* [Compose UI for Android](https://developer.android.com/jetpack/compose)
* [Android JetPack](https://developer.android.com/jetpack) - Collection of libraries that help you design robust, testable, and maintainable apps.
  * [Paging3](https://developer.android.com/topic/libraries/architecture/paging) - Load and display small chunks of data at a time.
  * [ViewModel](https://developer.android.com/topic/libraries/architecture/viewmodel) - Stores UI-related data that isn't destroyed on UI changes.
  * [Navigation Components](https://developer.android.com/jetpack/compose/navigation) - [Navigate between composables](https://funkymuse.dev/posts/compose_hilt_mm/)
  * [SavedStateHandle](https://developer.android.com/reference/androidx/lifecycle/SavedStateHandle) - A handle to saved state passed down to `ViewModel`. 
  * [Room](https://developer.android.google.cn/jetpack/androidx/releases/room) - Persistence library provides an abstraction layer over SQLite to allow for more robust database access while harnessing the full power of SQLite.
* [Dependency Injection](https://developer.android.com/training/dependency-injection) -
  * [Hilt-Dagger](https://dagger.dev/hilt/) - Standard library to incorporate Dagger dependency injection into an Android application.
  * [Hilt-ViewModel](https://developer.android.com/training/dependency-injection/hilt-jetpack) - DI for injecting `ViewModel`.

* Architecture
    * Clean Architecture
    * Multi module
    * MVVM


* Tests
  * TBA

## Find this repository useful? ❤️

Support it by joining [stargazers](https://github.com/FunkyMuse/Aurora/stargazers) for this repository. 🌠

And [follow me](https://github.com/FunkyMuse) or check my [blog](https://funkymuse.dev/) for my next creations! ⭐

## Contributions
To add a language that the app wasn't translated into, please open an issue!

Feature requests and translations are always welcome!

## License
[GNU General Public License v3.0](https://github.com/FunkyMuse/Aurora/blob/master/LICENSE)



